import os
from aiohttp import web
from utils import scanner,returnJson
# from utils import mapAppend
# from settings import OK_STATUS
from gbcall.settings import DEFAULT_PORT, ALIVE_SYMBOL
import importlib.util
import sys
from colorama import Fore

infoCache = {}
moduleCache = {}
funcCache = {}
# bindingCache = {}


def dynamicImport(path):
    spec = importlib.util.spec_from_file_location(
        path, path)
    foo = importlib.util.module_from_spec(spec)
    sys.modules[path] = foo
    spec.loader.exec_module(foo)
    return foo


async def handle(request):
    name = request.match_info.get('name', "Anonymous")
    text = "Hello, " + name
    return web.Response(text=text)


async def handshake(_request):
    return web.Response(text=ALIVE_SYMBOL["python"])


async def funcRegister(request):
    file = request.rel_url.query['filePath']
    base = request.rel_url.query['workingPath']

    absPath = os.path.join(base, file)
    try:
        module = dynamicImport(absPath)
    except:
        return returnJson(None,"no file or error in file")
    with open(os.path.join(base, file), 'r') as source:
        funcName, input, output = scanner(source.read())
        funcCache[funcName] = getattr(module,funcName)
        infoCache[funcName] = {"input": input, "output": output}
    return returnJson(funcName)



async def callFunc(request):
    data=await request.json()
    funcName = request.match_info.get('funcName', "Anonymous")
    res=funcCache[funcName](*data["args"])
    return returnJson(res)


def getFuncInfo(request):
    name = request.match_info.get('funcName')
    try:
        return returnJson(infoCache[name])
    except:
        return returnJson(None,"no such func")


app = web.Application()
app.add_routes([
    web.get('/isAlive', handshake),
    web.get('/funcRegister', funcRegister),
    web.post('/call/{funcName}', callFunc),
    web.get('/info/{funcName}', getFuncInfo),
    web.get('/b/{name}', handle)
])


print(f"python_server on: {Fore.RED}http://localhost:{DEFAULT_PORT}")
web.run_app(app, port=DEFAULT_PORT, print=None)



#! python3
import argparse
import os
from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientConnectorError
import asyncio
from gbcall.settings import DEFAULT_PORT, DESCRIPTION, ALIVE_SYMBOL
import urllib.parse
from colorama import Fore, Style
import subprocess
import sys


def prompt():
    parser = argparse.ArgumentParser(
        description=DESCRIPTION)

    parser.add_argument("filePath",
                        nargs='*', metavar="file_Pame", type=str,
                        help="The file containing function to be exposed.")

    args = parser.parse_args()

    if not args.filePath:
        print("No file path provided")
        return
    files = args.filePath

    targetServer = f'http://localhost:{DEFAULT_PORT}'

    async def main(reRun):
        if not reRun:
            return
        async with ClientSession() as session:
            try:
                async with session.get(targetServer+'/isAlive') as resp:
                    if (resp.status == 200 and
                            await resp.text() == ALIVE_SYMBOL["python"]):
                        for fileName in files:
                            params = urllib.parse.urlencode({
                                "filePath": fileName,
                                "workingPath": os.getcwd()
                            })
                            async with session.get(
                                f'{targetServer}/funcRegister?{params}'
                            ) as resp:
                                funcName = await resp.json()
                                if funcName['status'] == "ERR":
                                    print(f"{Fore.RED}ERROR:{Style.RESET_ALL}{funcName['res']}")
                                else:
                                    print(f"{Fore.BLUE}{funcName['res']}{Style.RESET_ALL} registered on {Fore.RED}{targetServer}{Style.RESET_ALL}")
            except ClientConnectorError:
                t=subprocess.Popen([sys.executable, os.path.split(os.path.realpath(__file__))[0]+'/server.py'],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.STDOUT)
                if not reRun:
                    asyncio.run(main(True))


    asyncio.run(main(True))


prompt()



import re
from aiohttp import web
import json

def scanner(code):
    regex = re.compile("@defineExpose\((.+)\)\ndef (.+):", re.S)
    IOStr = re.search(regex, code).group(1)
    SignStr = re.search(regex, code).group(2)
    args = re.search(re.compile(
        "input=\[(.+?)\]", re.S), IOStr).group(1).split(',')
    args = list(map(lambda s: s.strip(), args))
    ret = re.search(re.compile("output=\[(.+?)\]", re.S), IOStr).group(1).split(',')
    ret = list(map(lambda s: s.strip(), ret))
    funcName = re.search(re.compile("(.+?)\(.+?\)", re.S), SignStr).group(1)
    return funcName, args, ret


def mapAppend(m, key, val):
    if key in m:
        m.append(val)
    else:
        m[key] = [val]

def returnJson(data,error=None):
    if error:
        res={"res":error,"status":"ERR"}
    else:
        res={"res":data,"status":"OK"}
    return web.Response(
        text=json.dumps(res),
        content_type="application/json",
        headers={"Access-Control-Allow-Origin":"*"}
    )
